@@ -1,3 +1,4 @@
+from typing import Sequence, Dict, Any
 from .abstract import Manager, Model
 
 
@@ -10,24 +11,26 @@ class CloudwatchAlarmManager(Manager):
 
     service = 'cloudwatch'
 
-    def get(self, pk, **kwargs):
+    def get(self, pk: str, **kwargs) -> "CloudwatchAlarm":
         response = self.client.describe_alarms(AlarmNames=[pk])
         if 'MetricAlarms' in response and response['MetricAlarms']:
             return CloudwatchAlarm(response['MetricAlarms'][0])
         else:
             raise CloudwatchAlarm.DoesNotExist('No Cloudwatch Alarm with name "{}" exists in AWS'.format(pk))
 
-    def list(self, cluster, service):
-        response = self.client.describe_alarms(AlarmNamePrefix=['{}-{}'.format(cluster, service)])
+    def list(self, cluster: str, service: str, **kwargs) -> Sequence["CloudwatchAlarm"]:
+        response = self.client.describe_alarms(
+            AlarmNamePrefix=[f'{cluster}-{service}']
+        )
         if 'MetricAlarms' in response:
             return [CloudwatchAlarm(d) for d in response['MetricAlarms']]
         return []
 
-    def save(self, obj):
+    def save(self, obj: Model, **kwargs) -> None:
         self.delete(obj)
         self.client.put_metric_alarm(**obj.render_for_create())
 
-    def delete(self, obj):
+    def delete(self, obj: Model, **kwargs) -> None:
         try:
             self.client.delete_alarms(AlarmNames=[obj.pk])
         except self.client.exceptions.ResourceNotFound:
@@ -43,21 +46,21 @@ class CloudwatchAlarm(Model):
     objects = CloudwatchAlarmManager()
 
     @property
-    def pk(self):
+    def pk(self) -> str:
         return self.data['AlarmName']
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.data['AlarmName']
 
     @property
-    def arn(self):
+    def arn(self) -> str:
         return self.data.get('AlarmArn', None)
 
-    def set_policy_arn(self, arn):
+    def set_policy_arn(self, arn: str) -> None:
         self.data['AlarmActions'] = [arn]
 
-    def render_for_diff(self):
+    def render_for_diff(self) -> Dict[str, Any]:
         data = {}
         data['AlarmName'] = self.data['AlarmName']
         data['AlarmDescription'] = self.data['AlarmDescription']
@@ -70,4 +73,4 @@ class CloudwatchAlarm(Model):
         data['EvaluationPeriods'] = self.data['EvaluationPeriods']
         data['ComparisonOperator'] = self.data['ComparisonOperator']
         data['Threshold'] = self.data['Threshold']
-        return data
+        return data
\ No newline at end of file
